@@ -14,6 +14,7 @@ const Server = require('../topologies/server');
 const ServerSessionPool = require('../core').Sessions.ServerSessionPool;
 const emitDeprecationWarning = require('../utils').emitDeprecationWarning;
 const fs = require('fs');
+const WriteConcern = require('../write_concern');
 const BSON = require('../core/connection/utils').retrieveBSON();
 const CMAP_EVENT_NAMES = require('../cmap/events').CMAP_EVENT_NAMES;
 
@@ -105,6 +106,7 @@ const validOptionNames = [
   'w',
   'wtimeout',
   'j',
+  'writeConcern',
   'forceServerObjectId',
   'serializeFunctions',
   'ignoreUndefined',
@@ -290,7 +292,7 @@ function connect(mongoClient, url, options, callback) {
     const _finalOptions = createUnifiedOptions(object, options);
 
     // Check if we have connection and socket timeout set
-    if (_finalOptions.socketTimeoutMS == null) _finalOptions.socketTimeoutMS = 360000;
+    if (_finalOptions.socketTimeoutMS == null) _finalOptions.socketTimeoutMS = 0;
     if (_finalOptions.connectTimeoutMS == null) _finalOptions.connectTimeoutMS = 10000;
     if (_finalOptions.retryWrites == null) _finalOptions.retryWrites = true;
     if (_finalOptions.useRecoveryToken == null) _finalOptions.useRecoveryToken = true;
@@ -300,18 +302,16 @@ function connect(mongoClient, url, options, callback) {
       delete _finalOptions.db_options.auth;
     }
 
-    // `journal` should be translated to `j` for the driver
-    if (_finalOptions.journal != null) {
-      _finalOptions.j = _finalOptions.journal;
-      _finalOptions.journal = undefined;
-    }
-
     // resolve tls options if needed
     resolveTLSOptions(_finalOptions);
 
     // Store the merged options object
     mongoClient.s.options = _finalOptions;
 
+    // Apply read and write concern from parsed url
+    mongoClient.s.readPreference = ReadPreference.fromOptions(_finalOptions);
+    mongoClient.s.writeConcern = WriteConcern.fromOptions(_finalOptions);
+
     // Failure modes
     if (object.servers.length === 0) {
       return callback(new Error('connection string must contain at least one seed host'));
@@ -616,9 +616,12 @@ function createUnifiedOptions(finalOptions, options) {
     'mongos_options'
   ];
   const noMerge = ['readconcern', 'compression', 'autoencryption'];
+  const skip = ['w', 'wtimeout', 'j', 'journal', 'fsync', 'writeConcern'];
 
   for (const name in options) {
-    if (noMerge.indexOf(name.toLowerCase()) !== -1) {
+    if (skip.indexOf(name.toLowerCase()) !== -1) {
+      continue;
+    } else if (noMerge.indexOf(name.toLowerCase()) !== -1) {
       finalOptions[name] = options[name];
     } else if (childOptions.indexOf(name.toLowerCase()) !== -1) {
       finalOptions = mergeOptions(finalOptions, options[name], false);
@@ -636,6 +639,14 @@ function createUnifiedOptions(finalOptions, options) {
     }
   }
 
+  // Handle write concern keys separately, since `options` may have the keys at the top level or
+  // under `options.writeConcern`. The final merged keys will be under `finalOptions.writeConcern`.
+  // This way, `fromOptions` will warn once if `options` is using deprecated write concern options
+  const optionsWriteConcern = WriteConcern.fromOptions(options);
+  if (optionsWriteConcern) {
+    finalOptions.writeConcern = Object.assign({}, finalOptions.writeConcern, optionsWriteConcern);
+  }
+
   return finalOptions;
 }
 
@@ -760,12 +771,24 @@ function transformUrlOptions(_object) {
 
   if (object.wTimeoutMS) {
     object.wtimeout = object.wTimeoutMS;
+    object.wTimeoutMS = undefined;
   }
 
   if (_object.srvHost) {
     object.srvHost = _object.srvHost;
   }
 
+  // Any write concern options from the URL will be top-level, so we manually
+  // move them options under `object.writeConcern` to avoid warnings later
+  const wcKeys = ['w', 'wtimeout', 'j', 'journal', 'fsync'];
+  for (const key of wcKeys) {
+    if (object[key] !== undefined) {
+      if (object.writeConcern === undefined) object.writeConcern = {};
+      object.writeConcern[key] = object[key];
+      object[key] = undefined;
+    }
+  }
+
   return object;
 }
 
@@ -788,7 +811,7 @@ function translateOptions(options, translationOptions) {
   }
 
   // Set the socket and connection timeouts
-  if (options.socketTimeoutMS == null) options.socketTimeoutMS = 360000;
+  if (options.socketTimeoutMS == null) options.socketTimeoutMS = 0;
   if (options.connectTimeoutMS == null) options.connectTimeoutMS = 10000;
 
   if (!translationOptions.createServers) {
