@@ -12,6 +12,8 @@ const chai = require('chai');
 const expect = chai.expect;
 const sinon = require('sinon');
 const fs = require('fs');
+const os = require('os');
+const path = require('path');
 const crypto = require('crypto');
 const BSON = require('bson');
 const Long = BSON.Long;
@@ -1474,23 +1476,23 @@ describe('Change Streams', function() {
     }
   });
 
-  it.skip('should resume piping of Change Streams when a resumable error is encountered', {
+  it('should resume piping of Change Streams when a resumable error is encountered', {
     metadata: {
       requires: {
+        os: '!win32', // (fs.watch isn't reliable on win32)
         generators: true,
         topology: 'single',
         mongodb: '>=3.6'
       }
     },
     test: function(done) {
+      const filename = path.join(os.tmpdir(), '_nodemongodbnative_resumepipe.txt');
+      this.defer(() => fs.unlinkSync(filename));
       const configuration = this.configuration;
       const ObjectId = configuration.require.ObjectId;
       const Timestamp = configuration.require.Timestamp;
       const Long = configuration.require.Long;
 
-      // Contain mock server
-      let primaryServer = null;
-
       // Default message fields
       const defaultFields = {
         setName: 'rs',
@@ -1506,9 +1508,8 @@ describe('Change Streams', function() {
         hosts: ['localhost:32000', 'localhost:32001', 'localhost:32002']
       };
 
-      co(function*() {
-        primaryServer = yield mock.createServer();
-
+      mock.createServer(32000, 'localhost').then(primaryServer => {
+        this.defer(() => mock.cleanup());
         let counter = 0;
         primaryServer.setMessageHandler(request => {
           const doc = request.document;
@@ -1594,31 +1595,28 @@ describe('Change Streams', function() {
 
         client.connect((err, client) => {
           expect(err).to.not.exist;
+          this.defer(() => client.close());
 
           const database = client.db('integration_tests5');
           const collection = database.collection('MongoNetworkErrorTestPromises');
           const changeStream = collection.watch(pipeline);
 
-          const filename = '/tmp/_nodemongodbnative_resumepipe.txt';
-          const outStream = fs.createWriteStream(filename);
-
-          changeStream.stream({ transform: JSON.stringify }).pipe(outStream);
+          const outStream = fs.createWriteStream(filename, { flags: 'w' });
+          this.defer(() => outStream.close());
 
+          changeStream
+            .stream({ transform: change => JSON.stringify(change) + '\n' })
+            .pipe(outStream);
+          this.defer(() => changeStream.close());
           // Listen for changes to the file
-          const watcher = fs.watch(filename, function(eventType) {
-            assert.equal(eventType, 'change');
+          const watcher = fs.watch(filename, eventType => {
+            this.defer(() => watcher.close());
+            expect(eventType).to.equal('change');
 
             const fileContents = fs.readFileSync(filename, 'utf8');
-            const parsedFileContents = JSON.parse(fileContents);
-            assert.equal(parsedFileContents.fullDocument.a, 1);
-
-            watcher.close();
-
-            changeStream.close(err => {
-              expect(err).to.not.exist;
-
-              mock.cleanup(() => done());
-            });
+            const parsedFileContents = JSON.parse(fileContents.split(/\n/)[0]);
+            expect(parsedFileContents).to.have.nested.property('fullDocument.a', 1);
+            done();
           });
         });
       });
