@@ -1,10 +1,15 @@
 'use strict';
 
+const Topology = require('../../lib/core/sdam/topology').Topology;
 const test = require('./shared').assert;
 const setupDatabase = require('./shared').setupDatabase;
 const withMonitoredClient = require('./shared').withMonitoredClient;
-const expect = require('chai').expect;
+
 const ReadPreference = require('../../lib/core/topologies/read_preference');
+const withClient = require('./shared').withClient;
+const chai = require('chai');
+chai.use(require('chai-subset'));
+const expect = chai.expect;
 
 describe('ReadPreference', function() {
   before(function() {
@@ -654,4 +659,104 @@ describe('ReadPreference', function() {
       })
     });
   });
+
+  context('should enforce fixed primary read preference', function() {
+    const collectionName = 'ddl_collection';
+
+    beforeEach(function() {
+      const configuration = this.configuration;
+      const client = this.configuration.newClient(configuration.writeConcernMax(), {
+        useUnifiedTopology: true,
+        readPreference: 'primaryPreferred'
+      });
+      return withClient(client, (client, done) => {
+        const db = client.db(configuration.db);
+        db.addUser('default', 'pass', { roles: 'readWrite' }, () => {
+          db.createCollection('before_collection', () => {
+            db.createIndex(collectionName, { aloha: 1 }, done);
+          });
+        });
+      });
+    });
+
+    const methods = {
+      'Collection#createIndex': [{ quote: 'text' }],
+      'Db#createIndex': [collectionName, { quote: 'text' }],
+      'Db#addUser': ['thomas', 'pass', { roles: 'readWrite' }],
+      'Db#removeUser': ['default'],
+      'Db#createCollection': ['created_collection'],
+      'Db#dropCollection': ['before_collection'],
+      'Collection#dropIndex': ['aloha_1'],
+      'Collection#rename': ['new_name'],
+      'Db#dropDatabase': []
+    };
+
+    Object.keys(methods).forEach(operation => {
+      it(`${operation}`, {
+        metadata: {
+          requires: { topology: ['replicaset', 'sharded'] }
+        },
+        test: function() {
+          const configuration = this.configuration;
+          const client = this.configuration.newClient(configuration.writeConcernMax(), {
+            useUnifiedTopology: true,
+            readPreference: 'primaryPreferred'
+          });
+          return withClient(client, (client, done) => {
+            const db = client.db(configuration.db);
+            const args = methods[operation];
+            const split = operation.split('#');
+            const parentId = split[0];
+            const method = split[1];
+            const collection = db.collection(collectionName);
+            const parent = parentId === 'Collection' ? collection : parentId === 'Db' ? db : null;
+            const selectServerSpy = this.sinon.spy(Topology.prototype, 'selectServer');
+            const callback = err => {
+              expect(err).to.not.exist;
+              expect(selectServerSpy.called).to.equal(true);
+              if (typeof selectServerSpy.args[0][0] === 'function') {
+                expect(selectServerSpy)
+                  .nested.property('args[0][1].readPreference.mode')
+                  .to.equal(ReadPreference.PRIMARY);
+              } else {
+                expect(selectServerSpy)
+                  .nested.property('args[0][0].readPreference.mode')
+                  .to.equal(ReadPreference.PRIMARY);
+              }
+              done();
+            };
+            args.push(callback);
+            parent[method].apply(parent, args);
+          });
+        }
+      });
+    });
+  });
+
+  it('should respect readPreference from uri', {
+    metadata: { requires: { topology: 'replicaset', mongodb: '>=3.6' } },
+    test: withMonitoredClient('find', { queryOptions: { readPreference: 'secondary' } }, function(
+      client,
+      events,
+      done
+    ) {
+      expect(client.readPreference.mode).to.equal('secondary');
+      client
+        .db('test')
+        .collection('test')
+        .findOne({ a: 1 }, err => {
+          expect(err).to.not.exist;
+          expect(events)
+            .to.be.an('array')
+            .with.lengthOf(1);
+          expect(events[0]).to.containSubset({
+            commandName: 'find',
+            command: {
+              $readPreference: { mode: 'secondary' }
+            }
+          });
+          done();
+        });
+    })
+  });
 });
